==> grabutils-0.0.16-py3-none-any.whl: grabutils/bs/pageobj.py <==
import functools as fn

from bs4 import BeautifulSoup


class BsField:
    def __init__(self, selector, *then, src_attr='page', many=False):
        self.src_attr = src_attr
        self.selector = selector
        self.many = many
        self.then = then

    def __get__(self, obj, owner):
        page = obj.page()
        then = self.then

        if self.many:
            res = page.select(self.selector)
            return [
                fn.reduce(lambda v, func: func(v), then, r)
                for r in res
                if r
            ] if then else res

        else:
            res = page.select_one(self.selector)
            return fn.reduce(lambda v, func: func(v), then, res) if then and res else res


class BsPageObj:
    parser = 'lxml'
    _persist = False
    _enqueue = False

    def __init__(self, page: str):
        if isinstance(page, BeautifulSoup):
            self.__page = page
        else:
            self.__page = BeautifulSoup(str(page), features=self.parser)

    def page(self):
        return self.__page

    def as_dict(self):
        class_fields = set(
            fld
            for fld, inst
            in self.__class__.__dict__.items()
            if isinstance(inst, BsField)
        )

        return dict(
            (field, getattr(self, field))
            for field
            in dir(self)
            if
            (
                field in class_fields or
                field not in {'as_dict', 'page', 'persist', 'enqueue'}

            ) and
            not field.startswith('_') and
            not field[0].isupper() and
            not callable(getattr(self, field))
        )

    @classmethod
    def persist(cls) -> bool:
        return cls._persist

    def enqueue(self, func, **kwargs):
        pass


class Nested:
    def __init__(self, page_obj_clazz):
        self.page_obj_clazz = page_obj_clazz

    def __call__(self, value):
        return self.page_obj_clazz(value).as_dict()


def parse_href(node):
    return dict(
        href=node.attrs['href'],
        label=node.string.strip()
    )


def parse_img(node):
    return dict(
        src=node.attrs['src'],
        label=node.attrs.get('alt') if node else None
    )


def prepend(text: str):
    def wrapper(value: str):
        return text + value

    return wrapper


class Href(BsField):
    def __init__(self, selector, *then, **kwargs):
        super(Href, self).__init__(selector, parse_href, *then, **kwargs)


class Image(BsField):
    def __init__(self, selector, **kwargs):
        super(Image, self).__init__(selector, parse_img, **kwargs)

==> grabutils-0.0.19-py3-none-any.whl: grabutils/bs/pageobj.py <==
import functools as fn
import operator as op

from bs4 import BeautifulSoup, Tag


class BsField:
    def __init__(self, selector, *then, src_attr='page', many=False):
        self.src_attr = src_attr
        self.selector = selector
        self.many = many
        self.then = then

    def __call__(self, node):
        then = self.then

        if self.many:
            res = node.select(self.selector) if self.selector else node
            return [
                fn.reduce(lambda v, func: func(v), then, r)
                for r in res
                if r
            ] if then else res

        else:
            res = node.select_one(self.selector) if self.selector else node
            return fn.reduce(lambda v, func: func(v), then, res) if then and res else res

    def __get__(self, obj, owner):
        if not obj or not isinstance(obj, BsPageObj):
            return self

        page = obj.page()
        then = self.then

        return self.__call__(page)


class BsPageObj:
    _parser = 'lxml'
    _persist = False
    _enqueue = False

    def __init__(self, page: str):
        if isinstance(page, BeautifulSoup) or isinstance(page, Tag):
            self.__page = page
        else:
            self.__page = BeautifulSoup(str(page), features=self._parser)

    def page(self):
        return self.__page

    def as_dict(self):
        class_fields = set(
            fld
            for fld, inst
            in self.__class__.__dict__.items()
            if isinstance(inst, BsField)
        )

        return dict(
            (field, getattr(self, field))
            for field
            in dir(self)
            if
            (
                field in class_fields or
                field not in {'as_dict', 'page', 'persist', 'enqueue'}

            ) and
            not field.startswith('_') and
            not field[0].isupper() and
            not callable(getattr(self, field))
        )

    @classmethod
    def persist(cls) -> bool:
        return cls._persist

    def enqueue(self, func, **kwargs):
        pass


class Nested:
    def __init__(self, page_obj_clazz):
        self.page_obj_clazz = page_obj_clazz

    def __call__(self, value):
        return self.page_obj_clazz(value).as_dict()


inner_text = op.attrgetter('string')
stripped = op.methodcaller('strip')


def as_attr(attr):
    def as_attr_wrapper(node):
        return node.attrs[attr]

    return as_attr_wrapper


def parse_href(node):
    return dict(
        href=node.attrs['href'],
        label=node.string.strip()
    )


def parse_img(node):
    return dict(
        src=node.attrs['src'],
        label=node.attrs.get('alt') if node else None
    )


def prepend(text: str):
    def wrapper(value: str):
        return text + value

    return wrapper


class Href(BsField):
    def __init__(self, selector, *then, **kwargs):
        super(Href, self).__init__(selector, parse_href, *then, **kwargs)


class Image(BsField):
    def __init__(self, selector, **kwargs):
        super(Image, self).__init__(selector, parse_img, **kwargs)

class List:
    def __init__(self, field: BsField):
        self.field = field

    def __call__(self, node):
        return self.field.__call__(node)
